--
-- loader from ExtandedPlayerTeleport
--
-- @author  nilBrain @ VertexDezign
-- @date	30/11/2023
-- @node
--
-- Copyright (C) VertexDezign, Confidential, All Rights Reserved.

local modName = g_currentModName;
local modDir = g_currentModDirectory;
local modEnvironment;

local function visitPlace(self, superFunc)
	local distance = 0;
	local x, y, z = self.currentHotspot:getTeleportWorldPosition();
	local pX, pY, pZ = getWorldTranslation(g_currentMission.player.rootNode);

	distance = MathUtil.vector3Length(x-pX, y-pY, z-pZ);

	local function teleport()
		if g_currentMission.controlledVehicle ~= nil then
			g_currentMission:onLeaveVehicle(x, y, z, true, false);
		else
			g_currentMission.player:moveToAbsolute(x, y, z, false, false);
		end;
	end;
	modEnvironment:addDelayFunctionEvent(teleport, self, distance);
end;

local function enterVehicle(self, superFunc)
	local vehicle = InGameMenuMapUtil.getHotspotVehicle(self.currentHotspot);

	if vehicle ~= nil and vehicle.getIsEnterableFromMenu ~= nil and vehicle:getIsEnterableFromMenu() then
		local x, y, z = getWorldTranslation(vehicle.rootNode);
		local pX, pY, pZ = getWorldTranslation(g_currentMission.player.rootNode);

		local distance = MathUtil.vector3Length(x-pX, y-pY, z-pZ);

		local function teleport()
			if g_currentMission.controlledVehicle ~= nil then
				g_currentMission:onLeaveVehicle(x, y, z, true, false);
			else
				g_currentMission.player:moveToAbsolute(x, y, z, false, false);
			end;
		end;
		modEnvironment:addDelayFunctionEvent(teleport, self, distance);
	end;
end;


local function onSwitchVehicle(mission, superFunc, ...)
	if not mission.isPlayerFrozen and mission.isRunning then
		mission:showBlinkingWarning(g_i18n:getText("functionDesabledUseMenu"), 1500);
	end;
end;

InGameMenuMapFrame.onClickVisitPlace = Utils.overwrittenFunction(InGameMenuMapFrame.onClickVisitPlace, visitPlace);
InGameMenuMapFrame.onClickEnterVehicle = Utils.overwrittenFunction(InGameMenuMapFrame.onClickEnterVehicle, enterVehicle);
BaseMission.onSwitchVehicle = Utils.overwrittenFunction(BaseMission.onSwitchVehicle, onSwitchVehicle);


local loader = {};

function loader:loadMap()
	modEnvironment = ExtendedPlayerTeleport.new(modName, modDir);
	if modEnvironment.isClient then
		modEnvironment:loadSamples();
	end;

	g_currentMission.inGameMenu.pageMapOverview.buttonVisitPlace.onClickCallback = visitPlace;
	g_currentMission.inGameMenu.pageMapOverview.buttonEnterVehicle.onClickCallback = enterVehicle;

	if g_currentMission.extendedPlayerTeleport == nil then
		g_currentMission.extendedPlayerTeleport = modEnvironment;
	end;
end;


function loader:update(dt)
	modEnvironment:update(dt);
end;


addModEventListener(loader);